@@ -1,9 +1,15 @@
 # coding=utf-8
 import base64
+import fnmatch
+import glob
 import logging
 
 __author__ = 'ThucNC'
 
+import os
+
+import re
+
 from unidecode import unidecode
 
 _logger = logging.getLogger(__name__)
@@ -38,4 +44,15 @@ def base64_encode(data: str) -> str:
 
 
 def base64_decode(data: str) -> str:
-    return base64.b64decode(data.encode('utf-8')).decode('utf-8')
\ No newline at end of file
+    return base64.b64decode(data.encode('utf-8')).decode('utf-8')
+
+
+def findfiles(pattern, folder='.'):
+    """
+    Returns list of filenames from `where` path matched by 'which'
+    shell pattern. Matching is case-insensitive.
+    """
+
+    # TODO: recursive param with walk() filtering
+    rule = re.compile(fnmatch.translate(pattern), re.IGNORECASE)
+    return [os.path.join(folder, name) for name in os.listdir(folder) if rule.match(name)]
\ No newline at end of file
