@@ -98,7 +98,7 @@ class Problem:
         self.input_format = self.constraints = self.output_format = ""
         self.judge_mode = JudgeMode.oj
         self.xp = 100
-        self.testcase_format = "ucode_single_file"
+        self.testcase_format = "ucode"
         self.testcases_sample = []
         self.stock_testcases_sample = []
         self.testcases = []
