@@ -4,17 +4,19 @@ import json
 import os
 import codecs
 import re
+import shutil
 from builtins import staticmethod
 from shutil import copyfile
 from typing import List, Tuple
 
 from jinja2 import Template
 from ucode.helpers.clog import CLog
-from ucode.helpers.misc import make_problem_code, join_lines, make_slug
+from ucode.helpers.misc import make_problem_code, join_lines, findfiles
 from ucode.models.problem import Problem, TestCase
+from ucode.services.common import find_section
 
 
-def read_testcases_from_file(testcase_file):
+def read_ucode_testcases_from_file(testcase_file):
     count = 0
     inputi = ""
     outputi = ""
@@ -22,7 +24,7 @@ def read_testcases_from_file(testcase_file):
 
     testcases = []
 
-    with open(testcase_file, 'r') as fi:
+    with open(testcase_file, 'r', encoding="utf-8") as fi:
         for line in fi:
             if line.startswith("###"):
 
@@ -49,23 +51,6 @@ def read_testcases_from_file(testcase_file):
     return testcases
 
 
-def find_section(pattern, lines, start_index=0, once=False):
-    indices = []
-    content = {}
-    for i in range(start_index, len(lines)):
-        if re.match(pattern, lines[i], re.I):
-            indices.append(i)
-            for j in range(i + 1, len(lines)):
-                if lines[j].startswith('#'):
-                    break
-                content.setdefault(i, [])
-                content[i].append(lines[j])
-            if once:
-                break
-
-    return indices, content
-
-
 def fix_section_title(name, pattern, lines, replace, once=True):
     found = False
     for i in range(len(lines)):
@@ -147,7 +132,7 @@ class ProblemService:
         if not os.path.isfile(testcase_manual_file):
             CLog.warn(f"Manual testcases file `testcases_manual.txt` is missing!")
 
-        with open(statement_file) as fi:
+        with open(statement_file, encoding="utf-8") as fi:
             statement = fi.read()
             # print(statement)
             lines = statement.splitlines()
@@ -275,7 +260,7 @@ class ProblemService:
                     statement_file_bak = os.path.join(problem_folder, f"{problem_code}_bak{i}.md")
                     i += 1
                 copyfile(statement_file, statement_file_bak)
-                with open(statement_file, 'w') as f:
+                with open(statement_file, 'w', encoding="utf-8") as f:
                     f.write('\n'.join(lines))
         return problem_code
 
@@ -348,7 +333,7 @@ class ProblemService:
             CLog.warn(f"Sample testcases file `{problem_code}_testcases_sample.txt` is missing!")
 
         metadata = None
-        with open(metadata_file) as f:
+        with open(metadata_file, encoding="utf-8") as f:
             metadata = json.load(f)
 
         if not metadata:
@@ -361,7 +346,7 @@ class ProblemService:
             if not metadata.get("statement_language"):
                 CLog.error(f"`statement_language` is not specified in `{problem_code}_meta.json` file")
 
-        with open(statement_file) as fi:
+        with open(statement_file, encoding="utf-8") as fi:
             statement = fi.read()
             lines = statement.splitlines()
 
@@ -450,9 +435,12 @@ $$R = B^P \mod M$$"""
 
     @staticmethod
     def _parse_statement_file(statement_file, problem: Problem):
-        with open(statement_file) as fi:
+        with open(statement_file, encoding="utf-8") as fi:
             statement = fi.read()
             lines = statement.splitlines()
+            lines.append("")
+            lines.append("")
+            lines.append("")
 
             statement_i, statement_c = find_section('#\s+.*', lines)
             if statement_i:
@@ -489,11 +477,11 @@ $$R = B^P \mod M$$"""
                     if len(explanation_i) > i:
                         testcase.explanation = join_lines(explanation_c[explanation_i[i]]).strip('`').strip()
 
-                    problem.testcases.append(testcase)
+                    problem.testcases_sample.append(testcase)
 
     @staticmethod
     def _parse_statement_file_v1(statement_file, problem: Problem):
-        with open(statement_file) as fi:
+        with open(statement_file, encoding="utf-8") as fi:
             statement = fi.read()
             lines = statement.splitlines()
 
@@ -572,7 +560,7 @@ $$R = B^P \mod M$$"""
                     if len(explanation_i) > i:
                         testcase.explanation = join_lines(explanation_c[explanation_i[i]]).strip('`').strip()
 
-                    problem.testcases.append(testcase)
+                    problem.testcases_sample.append(testcase)
 
     @staticmethod
     def load(problem_folder, load_testcase=False):
@@ -618,7 +606,7 @@ $$R = B^P \mod M$$"""
             if editorial_prob.statement.strip():
                 problem.editorial = editorial_prob.statement
             else:
-                with open(problem_files["editorial"]) as fi:
+                with open(problem_files["editorial"], encoding="utf-8") as fi:
                     problem.editorial = fi.read()
 
         ProblemService._parse_statement_file(problem_files["statement"], problem)
@@ -634,14 +622,14 @@ $$R = B^P \mod M$$"""
 
         solution_file = os.path.join(problem_folder, f"{problem_code}.py")
         if os.path.isfile(solution_file):
-            with open(solution_file) as f:
+            with open(solution_file, encoding="utf-8") as f:
                 problem.solution = f.read()
 
         additional_solution_files = glob.glob(os.path.join(problem_folder, f"{problem_code}_solution.*"))
         for file_path in additional_solution_files:
             sub_path = os.path.join(problem_folder, f"{problem_code}_solution")
             idx = file_path.find(sub_path) + len(sub_path)
-            with open(file_path) as f:
+            with open(file_path, encoding="utf-8") as f:
                 problem.solutions.append({"lang": file_path[idx + 1:],
                                           "code": f.read()})
         # print(*problem.solutions, sep="\n")
@@ -651,31 +639,18 @@ $$R = B^P \mod M$$"""
             file_path = testcase_generators[0]
             if os.path.exists(file_path):
                 idx = file_path.rfind(".")
-                with open(file_path) as f:
+                with open(file_path, encoding="utf-8") as f:
                     problem.testcase_generator = {"lang": file_path[idx + 1:], "code": f.read()}
             # print(problem.testcase_generator)
 
-        problem.testcases = []
         if load_testcase:
-            file_names = ["testcases_sample_stock.txt", "testcases_sample.txt", "testcases_stock.txt", "testcases.txt"]
-            input_set = set()
-            for file_name in file_names:
-                testcase_file = os.path.abspath(os.path.join(problem_folder, f"{problem_code}_{file_name}"))
+            if problem.testcase_format == "ucode":
+                ProblemService.load_ucode_testcases(problem_folder, problem)
+            elif problem.testcase_format == "themis":
+                ProblemService.load_themis_testcases(problem_folder, problem)
+            else:
+                CLog.error(f"Unknown testcase format {problem.testcase_format}")
 
-                if not os.path.exists(testcase_file):
-                    CLog.info(f'`{testcase_file}` file not existed, skipping...')
-                else:
-                    tests = read_testcases_from_file(testcase_file)
-                    # print(f"reading testcases in file {testcase_file}: {len(tests)}")
-                    for t in tests:
-                        _input, _output = t['input'], t['output']
-                        # print(f"Input: {input}, output: {output}")
-                        if _input not in input_set:
-                            problem.testcases.append(TestCase(input=_input, output=_output))
-                        if "_sample" in file_name:
-                            problem.testcases_sample.append(TestCase(input=_input, output=_output))
-                        input_set.add(_input)
-                        # print(input_set)
             print(f"Loaded {len(problem.testcases)} testcases")
 
         if not problem.testcases_sample and problem.testcases:
@@ -707,7 +682,7 @@ $$R = B^P \mod M$$"""
             if editorial_prob.statement.strip():
                 problem.editorial = editorial_prob.statement
             else:
-                with open(editorial_file) as fi:
+                with open(editorial_file, encoding="utf-8") as fi:
                     problem.editorial = fi.read()
 
         ProblemService._parse_statement_file_v1(statement_file, problem)
@@ -723,14 +698,14 @@ $$R = B^P \mod M$$"""
 
         solution_file = os.path.join(problem_folder, f"{problem_code}.py")
         if os.path.isfile(solution_file):
-            with open(solution_file) as f:
+            with open(solution_file, encoding="utf-8") as f:
                 problem.solution = f.read()
 
         additional_solution_files = glob.glob(os.path.join(problem_folder, f"solution.*"))
         for file_path in additional_solution_files:
             sub_path = os.path.join(problem_folder, "solution")
             idx = file_path.find(sub_path) + len(sub_path)
-            with open(file_path) as f:
+            with open(file_path, encoding="utf-8") as f:
                 problem.solutions.append({"lang": file_path[idx + 1:],
                                           "code": f.read()})
         # print(*problem.solutions, sep="\n")
@@ -740,7 +715,7 @@ $$R = B^P \mod M$$"""
             file_path = testcase_generators[0]
             if os.path.exists(file_path):
                 idx = file_path.rfind(".")
-                with open(file_path) as f:
+                with open(file_path, encoding="utf-8") as f:
                     problem.testcase_generator = {"lang": file_path[idx + 1:], "code": f.read()}
             # print(problem.testcase_generator)
 
@@ -754,7 +729,7 @@ $$R = B^P \mod M$$"""
                 if not os.path.exists(testcase_file):
                     CLog.warn(f'`{testcase_file}` file not existed, skipping...')
                 else:
-                    tests = read_testcases_from_file(testcase_file)
+                    tests = read_ucode_testcases_from_file(testcase_file)
                     # print(f"reading testcases in file {testcase_file}: {len(tests)}")
                     for t in tests:
                         _input, _output = t['input'], t['output']
@@ -803,10 +778,10 @@ $$R = B^P \mod M$$"""
         if problem.testcases:
             testcases += problem.testcases
 
-        with open(os.path.join(template_path, 'testcases.txt.j2')) as file_:
+        with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
             template = Template(file_.read())
             content = template.render(testcases=testcases)
-            f = open(problem_folder + f"/{problem_code}_testcases.txt", 'w')
+            f = open(problem_folder + f"/{problem_code}_testcases.txt", 'w', encoding="utf-8")
             f.write(content)
             f.close()
 
@@ -816,32 +791,39 @@ $$R = B^P \mod M$$"""
         if problem.stock_testcases_sample:
             sample_testcases += problem.stock_testcases_sample
 
-        with open(os.path.join(template_path, 'testcases.txt.j2')) as file_:
+        with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
             template = Template(file_.read())
             content = template.render(testcases=problem.testcases_sample)
-            f = open(problem_folder + f"/{problem_code}_testcases_sample.txt", 'w')
+            f = open(problem_folder + f"/{problem_code}_testcases_sample.txt", 'w', encoding="utf-8")
             f.write(content)
             f.close()
 
+        if problem.testcase_format == "themis" and overwrite:
+            # clear all themis testcases
+            test_folders = glob.glob(os.path.join(problem_folder, "Test*"))
+            for test_folder in test_folders:
+                shutil.rmtree(test_folder)
+
         problem.testcases_sample = sample_testcases
 
         if lang:
             problem.statement_language = lang
         print("src", problem.src_url)
 
-        with open(os.path.join(template_path, 'meta.json.j2')) as file_:
+        problem.testcase_format = "ucode"
+        with open(os.path.join(template_path, 'meta.json.j2'), encoding="utf-8") as file_:
             template = Template(file_.read())
             meta = template.render(problem=problem,
                                    statement_translations=json.dumps(list(problem.translations.keys())),
                                    tags=json.dumps(problem.tags))
-            f = codecs.open(problem_folder + ("/%s_meta.json" % problem_code), "w", "utf-8")
+            f = codecs.open(problem_folder + ("/%s_meta.json" % problem_code), "w", encoding="utf-8")
             f.write(meta)
             f.close()
 
-        with open(os.path.join(template_path, 'statement.md.j2')) as file_:
+        with open(os.path.join(template_path, 'statement.md.j2'), encoding="utf-8") as file_:
             template = Template(file_.read())
             statement = template.render(problem=problem)
-            f = codecs.open(problem_folder + ("/%s.md" % problem_code), "w", "utf-8")
+            f = codecs.open(problem_folder + ("/%s.md" % problem_code), "w", encoding="utf-8")
             f.write(statement)
             f.close()
 
@@ -849,47 +831,48 @@ $$R = B^P \mod M$$"""
             with open(os.path.join(template_path, 'statement.md.j2')) as file_:
                 template = Template(file_.read())
                 statement = template.render(problem=prob_tran)
-                f = codecs.open(problem_folder + (f"/{problem_code}_tran_{lang}.md"), "w", "utf-8")
+                f = codecs.open(problem_folder + (f"/{problem_code}_tran_{lang}.md"), "w", encoding="utf-8")
                 f.write(statement)
                 f.close()
 
-        with open(os.path.join(template_path, 'editorial.md.j2')) as file_:
+        with open(os.path.join(template_path, 'editorial.md.j2'), encoding="utf-8") as file_:
             template = Template(file_.read())
             if not problem.editorial:
                 problem.editorial = ""
             statement = template.render(problem=problem)
-            f = codecs.open(problem_folder + ("/%s_editorial.md" % problem_code), "w", "utf-8")
+            f = codecs.open(problem_folder + ("/%s_editorial.md" % problem_code), "w", encoding="utf-8")
             f.write(statement)
             f.close()
 
         if not problem.solution:
-            with open(os.path.join(template_path, 'solution.py.j2')) as file_:
+            with open(os.path.join(template_path, 'solution.py.j2'), encoding="utf-8") as file_:
                 template = Template(file_.read())
                 content = template.render(problem_code=problem_code,
                                           solution=problem.solution if problem.solution else "pass")
-                f = open(problem_folder + (f"/{problem_code}.py"), 'w')
+                f = open(problem_folder + (f"/{problem_code}.py"), 'w', encoding="utf-8")
                 f.write(content)
                 f.close()
         else:
-            f = open(problem_folder + (f"/{problem_code}.py"), 'w')
+            f = open(problem_folder + (f"/{problem_code}.py"), 'w', encoding="utf-8")
             f.write(problem.solution)
             f.close()
 
         if not problem.testcase_generator:
-            with open(os.path.join(template_path, 'generator.py.j2')) as file_:
+            with open(os.path.join(template_path, 'generator.py.j2'), encoding="utf-8") as file_:
                 template = Template(file_.read())
                 content = template.render(problem_code=problem_code)
-                f = open(problem_folder + (f"/{problem_code}_generator.py"), 'w')
+                f = open(problem_folder + (f"/{problem_code}_generator.py"), 'w', encoding="utf-8")
                 f.write(content)
                 f.close()
         else:
-            f = open(f"{problem_folder}/{problem_code}_generator.{problem.testcase_generator['lang']}", 'w')
+            f = open(f"{problem_folder}/{problem_code}_generator.{problem.testcase_generator['lang']}",
+                     'w', encoding="utf-8")
             f.write(problem.testcase_generator["code"])
             f.close()
 
         if problem.solutions:
             for solution in problem.solutions:
-                f = open(f"{problem_folder}/{problem_code}_solution.{solution['lang']}", 'w')
+                f = open(f"{problem_folder}/{problem_code}_solution.{solution['lang']}", 'w', encoding="utf-8")
                 f.write(solution["code"])
                 f.close()
 
@@ -897,6 +880,55 @@ $$R = B^P \mod M$$"""
 
         CLog.important(f'Problem created at `{problem_folder}`')
 
+    @staticmethod
+    def load_ucode_testcases(problem_folder, problem: Problem):
+        problem.testcases = []
+        file_names = ["testcases_sample_stock.txt", "testcases_sample.txt", "testcases_stock.txt", "testcases.txt"]
+        input_set = set()
+        for file_name in file_names:
+            testcase_file = os.path.abspath(os.path.join(problem_folder, f"{problem.code}_{file_name}"))
+
+            if not os.path.exists(testcase_file):
+                CLog.info(f'`{testcase_file}` file not existed, skipping...')
+            else:
+                tests = read_ucode_testcases_from_file(testcase_file)
+                # print(f"reading testcases in file {testcase_file}: {len(tests)}")
+                for t in tests:
+                    _input, _output = t['input'], t['output']
+                    # print(f"Input: {input}, output: {output}")
+                    if _input not in input_set:
+                        problem.testcases.append(TestCase(input=_input, output=_output))
+                    if "_sample" in file_name:
+                        problem.testcases_sample.append(TestCase(input=_input, output=_output))
+                    input_set.add(_input)
+                    # print(input_set)
+
+    @staticmethod
+    def load_themis_testcases(problem_folder, problem: Problem):
+        problem.testcases = []
+        test_folders = glob.glob(os.path.join(problem_folder, "Test*"))
+        for test_folder in test_folders:
+            # test_folder = os.path.abspath(test_folder)
+            if not os.path.isdir(test_folder):
+                continue
+            # print(test_folder)
+
+            input_files = findfiles("*.INP", test_folder)
+            output_files = findfiles("*.OUT", test_folder)
+            # print(input_files, len(input_files))
+            # print(output_files, len(output_files))
+            if len(input_files) != 1 or len(output_files) != 1:
+                CLog.error(f"Invalid testcase folder `{os.path.abspath(test_folder)}`")
+                continue
+            input_file = input_files[0]
+            output_file = output_files[0]
+
+            with open(input_file, 'r', encoding="utf-8") as fi:
+                _input = fi.read()
+            with open(output_file, 'r', encoding="utf-8") as fo:
+                _output = fo.read()
+            problem.testcases.append(TestCase(input=_input, output=_output))
+
 
 if __name__ == "__main__":
     # DsaProblem.create_problem('../../problems', 'Counting Sort 3', lang="en",
@@ -909,12 +941,16 @@ if __name__ == "__main__":
     #                       translations=['vi'], load_testcase=True)
     # problem = DsaProblem.load_v1('/home/thuc/projects/ucode/weekly-algorithm-problems/week03/p02_key_races',
     #                       translations=['vi'], load_testcase=True)
-    problem = ProblemService.load('/home/thuc/projects/ucode/ucode-cli/ucode/problems/p02_key_races', load_testcase=True)
+    problem = ProblemService.load('../../../problems/bridge_passing', load_testcase=True)
+    # problem = ProblemService.load('../../../problems/clique_in_the_divisibility_graph', load_testcase=True)
     print(problem.name)
+    print(problem.testcase_format)
+    print(len(problem.testcases))
+    print(len(problem.testcases_sample))
     # print(problem.translations['vi'].statement)
     # print(problem.editorial)
-    # DsaProblem.save(problem, "../../problems", overwrite=True, problem_folder="p02_key_races")
-    ProblemService.save(problem, "../../problems", overwrite=True)
+    ProblemService.save(problem, "../../../problems", problem_folder="bridge_passing2", overwrite=True)
+    # ProblemService.save(problem, "../../../problems", overwrite=True)
 
     # load_problem('/home/thuc/teko/online-judge/dsa-problems/number_theory/num001_sumab')
     # load_problem('/home/thuc/teko/online-judge/dsa-problems/unsorted/minhhhh/m010_odd_to_even')
